Attribute VB_Name = "modXaml"
Option Explicit

' Programmed by Jason Peter Brown (jbrown@statslog.com)
' Public Domain, no warranty, use at own risk!


Private Declare Function GetSysColor Lib "user32" (ByVal nIndex As Long) As Long

Public Function xamlOleColorToARGB(ByVal p_Color As OLE_COLOR, ByVal p_TransparencyPercent As Single) As String
   Dim r As Byte  ' Red channel
   Dim b As Byte  ' Blue channel
   Dim G As Byte  ' Green channel
   Dim A As Byte  ' Alpha/Transparency
   
   If p_TransparencyPercent < 0 Or p_TransparencyPercent > 100 Then
      ' Invalid transparency percentage.
      Err.Raise 5 ' Invalid procedure call or argument
   End If
   
   ' Calculate alpha/transparency byte value
   A = CByte(255 - 255 * (p_TransparencyPercent / 100))
   
   ' Convert system colour to Long Integer RGB colour
   p_Color = GetActualColor(p_Color)
   
   ' Convert Long Integer RGB colour to individual R, G and B bytes
   r = p_Color And &HFF&
   G = (p_Color And &HFF00&) \ &H100&
   b = (p_Color And &HFF0000) \ &H10000
   
   ' Return a formatted XAML Hex string for the colour (for example, #80000000 = Black @ 50% transparency
   xamlOleColorToARGB = "#" & LongToHex(A, 2) & LongToHex(r, 2) & LongToHex(G, 2) & LongToHex(b, 2)
End Function

Private Function GetActualColor(ByVal p_Color As Long) As Long
   ' This function returns an actual color for the passed RGB colour represented as a Long Integer
   ' This means that passed system colors (e.g. vbButtonFace) will return a useable RGB color as a long value
   ' Non-system colors that are passed will be returned unmodified, so it is recommended that you
   ' always call this function when dealing with colors from an unknown source (database, user-defined, etc...)
   
   If p_Color And &H80000000 Then
      p_Color = GetSysColor(p_Color And &HFFFFFF)
   End If
   
   GetActualColor = p_Color
End Function

Private Function LongToHex(ByVal p_Number As Long, Optional ByVal p_MinLen As Integer = 2) As String
   Dim l_Hex As String
   Dim l_Len As Long
   
   If p_MinLen < 1 Or p_MinLen > 8 Then
      Err.Raise 5 ' Invalid number of characters
   End If
   
   l_Hex = Hex$(p_Number)
   l_Len = Len(l_Hex)
   If l_Len > p_MinLen Then
      ' The length of the hex number is longer than the minimum length
      ' so set the minimum length to the length of the hex string
      ' (otherwise the number will be trimmed and incorrect).
      p_MinLen = l_Len
   End If
   
   LongToHex = Right$("00000000" & l_Hex, p_MinLen)
End Function

